namespace ReportLibrary
{
    using System;
    using Telerik.Reporting;
    using Telerik.Reporting.Expressions;

    public partial class SampleReport : Report
    {
        public SampleReport()
        {
            this.InitializeComponent();

            this.crosstabDataSource.DataSource = new SampleData[]
            {
                new SampleData("Subject 1", "Result 1", 1),
                new SampleData("Subject 1", "Result 2", 1),
                new SampleData("Subject 1", "Result 2", 1),
                new SampleData("Subject 1", "Result 3", 1),
                new SampleData("Subject 1", "Result 3", 2),
                new SampleData("Subject 2", "Result 1", 1),
                new SampleData("Subject 2", "Result 1", 2),
                new SampleData("Subject 2", "Result 1", 3),
                new SampleData("Subject 2", "Result 2", 2),
                new SampleData("Subject 2", "Result 2", 3),
                new SampleData("Subject 2", "Result 3", 2),
                new SampleData("Subject 2", "Result 3", 2),
                new SampleData("Subject 3", "Result 1", 2),
                new SampleData("Subject 3", "Result 1", 3),
                new SampleData("Subject 3", "Result 1", 4),
                new SampleData("Subject 3", "Result 2", 3),
                new SampleData("Subject 3", "Result 2", 4),
                new SampleData("Subject 3", "Result 3", 1),
                new SampleData("Subject 3", "Result 3", 3),
                new SampleData("Subject 3", "Result 3", 4)
            };

            ReportParameter sortParameter = this.ReportParameters["SortBy"];

            sortParameter.AvailableValues.DataSource = new string[]
            {
                "Subject 1",
                "Subject 2",
                "Subject 3"
            };
        }
    }

    public class SampleData
    {
        private string subject;
        private string result;
        private int value;

        public string Subject
        {
            get { return this.subject; }
        }

        public string Result
        {
            get { return this.result; }
        }

        public int Value
        {
            get { return this.value; }
        }

        public SampleData(string subject,
                          string result,
                          int value)
        {
            this.subject = subject;
            this.result = result;
            this.value = value;
        }
    }

    [AggregateFunction(Name = "SliceSum", Description = "Calculates the sum of values that match a certain criteria.")]
    public class SliceSum : IAggregateFunction
    {
        int totalValue;

        public void Init()
        {
            this.totalValue = 0;
        }

        public object GetValue()
        {
            return this.totalValue;
        }

        public void Accumulate(object[] values)
        {
            object filterValue = values[0];
            object slicedValue = values[1];

            if (object.Equals(filterValue, slicedValue))
            {
                object summedValue = values[2];

                if (null != summedValue)
                {
                    this.totalValue += Convert.ToInt32(summedValue);
                }
            }
        }

        public void Merge(IAggregateFunction aggregate)
        {
            SliceSum sliceSum = (SliceSum)aggregate;

            this.totalValue += sliceSum.totalValue;
        }
    }
}
